@@ -24,7 +24,7 @@ import (
 
 // Item is an item that appears in the list.
 type Item interface {
-	// Filter value is the value we use when filtering against this item when
+	// FilterValue is the value we use when filtering against this item when
 	// we're filtering the list.
 	FilterValue() string
 }
@@ -87,7 +87,7 @@ type Rank struct {
 // DefaultFilter uses the sahilm/fuzzy to filter through the list.
 // This is set by default.
 func DefaultFilter(term string, targets []string) []Rank {
-	var ranks = fuzzy.Find(term, targets)
+	ranks := fuzzy.Find(term, targets)
 	sort.Stable(ranks)
 	result := make([]Rank, len(ranks))
 	for i, r := range ranks {
@@ -99,6 +99,20 @@ func DefaultFilter(term string, targets []string) []Rank {
 	return result
 }
 
+// UnsortedFilter uses the sahilm/fuzzy to filter through the list. It does not
+// sort the results.
+func UnsortedFilter(term string, targets []string) []Rank {
+	ranks := fuzzy.FindNoSort(term, targets)
+	result := make([]Rank, len(ranks))
+	for i, r := range ranks {
+		result[i] = Rank{
+			Index:          r.Index,
+			MatchedIndexes: r.MatchedIndexes,
+		}
+	}
+	return result
+}
+
 type statusMessageTimeoutMsg struct{}
 
 // FilterState describes the current filtering state on the model.
@@ -132,8 +146,9 @@ type Model struct {
 	itemNameSingular string
 	itemNamePlural   string
 
-	Title  string
-	Styles Styles
+	Title             string
+	Styles            Styles
+	InfiniteScrolling bool
 
 	// Key mappings for navigating the list.
 	KeyMap KeyMap
@@ -183,18 +198,18 @@ type Model struct {
 func New(items []Item, delegate ItemDelegate, width, height int) Model {
 	styles := DefaultStyles()
 
-	sp := spinner.NewModel()
+	sp := spinner.New()
 	sp.Spinner = spinner.Line
 	sp.Style = styles.Spinner
 
-	filterInput := textinput.NewModel()
+	filterInput := textinput.New()
 	filterInput.Prompt = "Filter: "
 	filterInput.PromptStyle = styles.FilterPrompt
-	filterInput.CursorStyle = styles.FilterCursor
+	filterInput.Cursor.Style = styles.FilterCursor
 	filterInput.CharLimit = 64
 	filterInput.Focus()
 
-	p := paginator.NewModel()
+	p := paginator.New()
 	p.Type = paginator.Dots
 	p.ActiveDot = styles.ActivePaginationDot.String()
 	p.InactiveDot = styles.InactivePaginationDot.String()
@@ -221,7 +236,7 @@ func New(items []Item, delegate ItemDelegate, width, height int) Model {
 		items:     items,
 		Paginator: p,
 		spinner:   sp,
-		Help:      help.NewModel(),
+		Help:      help.New(),
 	}
 
 	m.updatePagination()
@@ -231,7 +246,7 @@ func New(items []Item, delegate ItemDelegate, width, height int) Model {
 
 // NewModel returns a new model with sensible defaults.
 //
-// Deprecated. Use New instead.
+// Deprecated: use [New] instead.
 var NewModel = New
 
 // SetFilteringEnabled enables or disables filtering. Note that this is different
@@ -260,7 +275,7 @@ func (m Model) ShowTitle() bool {
 	return m.showTitle
 }
 
-// SetShowFilter shows or hides the filer bar. Note that this does not disable
+// SetShowFilter shows or hides the filter bar. Note that this does not disable
 // filtering, it simply hides the built-in filter view. This allows you to
 // use the FilterInput to render the filtering UI differently without having to
 // re-implement filtering from scratch.
@@ -291,7 +306,7 @@ func (m Model) ShowStatusBar() bool {
 	return m.showStatusBar
 }
 
-// SetStatusBarItemName defines a replacement for the items identifier.
+// SetStatusBarItemName defines a replacement for the item's identifier.
 // Defaults to item/items.
 func (m *Model) SetStatusBarItemName(singular, plural string) {
 	m.itemNameSingular = singular
@@ -303,7 +318,7 @@ func (m Model) StatusBarItemName() (string, string) {
 	return m.itemNameSingular, m.itemNamePlural
 }
 
-// SetShowPagination hides or shoes the paginator. Note that pagination will
+// SetShowPagination hides or shows the paginator. Note that pagination will
 // still be active, it simply won't be displayed.
 func (m *Model) SetShowPagination(v bool) {
 	m.showPagination = v
@@ -331,7 +346,7 @@ func (m Model) Items() []Item {
 	return m.items
 }
 
-// Set the items available in the list. This returns a command.
+// SetItems sets the items available in the list. This returns a command.
 func (m *Model) SetItems(i []Item) tea.Cmd {
 	var cmd tea.Cmd
 	m.items = i
@@ -362,7 +377,7 @@ func (m *Model) ResetFilter() {
 	m.resetFiltering()
 }
 
-// Replace an item at the given index. This returns a command.
+// SetItem replaces an item at the given index. This returns a command.
 func (m *Model) SetItem(index int, item Item) tea.Cmd {
 	var cmd tea.Cmd
 	m.items[index] = item
@@ -375,8 +390,8 @@ func (m *Model) SetItem(index int, item Item) tea.Cmd {
 	return cmd
 }
 
-// Insert an item at the given index. If index is out of the upper bound, the
-// item will be appended. This returns a command.
+// InsertItem inserts an item at the given index. If the index is out of the upper bound,
+// the item will be appended. This returns a command.
 func (m *Model) InsertItem(index int, item Item) tea.Cmd {
 	var cmd tea.Cmd
 	m.items = insertItemIntoSlice(m.items, item, index)
@@ -404,7 +419,7 @@ func (m *Model) RemoveItem(index int) {
 	m.updatePagination()
 }
 
-// Set the item delegate.
+// SetDelegate sets the item delegate.
 func (m *Model) SetDelegate(d ItemDelegate) {
 	m.delegate = d
 	m.updatePagination()
@@ -418,7 +433,7 @@ func (m Model) VisibleItems() []Item {
 	return m.items
 }
 
-// SelectedItems returns the current selected item in the list.
+// SelectedItem returns the current selected item in the list.
 func (m Model) SelectedItem() Item {
 	i := m.Index()
 
@@ -459,6 +474,13 @@ func (m *Model) CursorUp() {
 
 	// If we're at the start, stop
 	if m.cursor < 0 && m.Paginator.Page == 0 {
+		// if infinite scrolling is enabled, go to the last item
+		if m.InfiniteScrolling {
+			m.Paginator.Page = m.Paginator.TotalPages - 1
+			m.cursor = m.Paginator.ItemsOnPage(len(m.VisibleItems())) - 1
+			return
+		}
+
 		m.cursor = 0
 		return
 	}
@@ -501,6 +523,12 @@ func (m *Model) CursorDown() {
 	}
 
 	m.cursor = itemsOnPage - 1
+
+	// if infinite scrolling is enabled, go to the first item
+	if m.InfiniteScrolling {
+		m.Paginator.Page = 0
+		m.cursor = 0
+	}
 }
 
 // PrevPage moves to the previous page, if available.
@@ -526,7 +554,7 @@ func (m Model) FilterValue() string {
 // SettingFilter returns whether or not the user is currently editing the
 // filter value. It's purely a convenience method for the following:
 //
-//     m.FilterState() == Filtering
+//	m.FilterState() == Filtering
 //
 // It's included here because it's a common thing to check for when
 // implementing this component.
@@ -537,8 +565,7 @@ func (m Model) SettingFilter() bool {
 // IsFiltered returns whether or not the list is currently filtered.
 // It's purely a convenience method for the following:
 //
-//	 m.FilterState() == FilterApplied
-//
+//	m.FilterState() == FilterApplied
 func (m Model) IsFiltered() bool {
 	return m.filterState == FilterApplied
 }
@@ -558,7 +585,7 @@ func (m *Model) SetSpinner(spinner spinner.Spinner) {
 	m.spinner.Spinner = spinner
 }
 
-// Toggle the spinner. Note that this also returns a command.
+// ToggleSpinner toggles the spinner. Note that this also returns a command.
 func (m *Model) ToggleSpinner() tea.Cmd {
 	if !m.showSpinner {
 		return m.StartSpinner()
@@ -570,7 +597,7 @@ func (m *Model) ToggleSpinner() tea.Cmd {
 // StartSpinner starts the spinner. Note that this returns a command.
 func (m *Model) StartSpinner() tea.Cmd {
 	m.showSpinner = true
-	return spinner.Tick
+	return m.spinner.Tick
 }
 
 // StopSpinner stops the spinner.
@@ -578,8 +605,8 @@ func (m *Model) StopSpinner() {
 	m.showSpinner = false
 }
 
-// Helper for disabling the keybindings used for quitting, in case you want to
-// handle this elsewhere in your application.
+// DisableQuitKeybindings is a helper for disabling the keybindings used for quitting,
+// in case you want to handle this elsewhere in your application.
 func (m *Model) DisableQuitKeybindings() {
 	m.disableQuitKeybindings = true
 	m.KeyMap.Quit.SetEnabled(false)
@@ -647,7 +674,7 @@ func (m Model) itemsAsFilterItems() filteredItems {
 			item: item,
 		}
 	}
-	return filteredItems(fi)
+	return fi
 }
 
 // Set keybindings according to the filter state.
@@ -1147,7 +1174,7 @@ func (m Model) populatedView() string {
 		if m.filterState == Filtering {
 			return ""
 		}
-		return m.Styles.NoItems.Render("No " + m.itemNamePlural + " found.")
+		return m.Styles.NoItems.Render("No " + m.itemNamePlural + ".")
 	}
 
 	if len(items) > 0 {
@@ -1191,11 +1218,11 @@ func filterItems(m Model) tea.Cmd {
 			return FilterMatchesMsg(m.itemsAsFilterItems()) // return nothing
 		}
 
-		targets := []string{}
 		items := m.items
+		targets := make([]string, len(items))
 
-		for _, t := range items {
-			targets = append(targets, t.FilterValue())
+		for i, t := range items {
+			targets[i] = t.FilterValue()
 		}
 
 		filterMatches := []filteredItem{}
