@@ -37,13 +37,12 @@ const (
 	defaultDamping   = 1.0
 )
 
-// Option is used to set options in NewModel. For example:
-//
-//     progress := NewModel(
-//	       WithRamp("#ff0000", "#0000ff"),
-//	       WithoutPercentage(),
-//     )
+// Option is used to set options in New. For example:
 //
+//	    progress := New(
+//		       WithRamp("#ff0000", "#0000ff"),
+//		       WithoutPercentage(),
+//	    )
 type Option func(*Model)
 
 // WithDefaultGradient sets a gradient fill with default colors.
@@ -97,7 +96,7 @@ func WithWidth(w int) Option {
 }
 
 // WithSpringOptions sets the initial frequency and damping options for the
-// progressbar's built-in spring-based animation. Frequency corresponds to
+// progress bar's built-in spring-based animation. Frequency corresponds to
 // speed, and damping to bounciness. For details see:
 //
 // https://github.com/charmbracelet/harmonica
@@ -137,7 +136,7 @@ type Model struct {
 	Full      rune
 	FullColor string
 
-	// "Empty" sections of progress bar.
+	// "Empty" sections of the progress bar.
 	Empty      rune
 	EmptyColor string
 
@@ -192,15 +191,15 @@ func New(opts ...Option) Model {
 
 // NewModel returns a model with default values.
 //
-// Deprecated. Use New instead.
+// Deprecated: use [New] instead.
 var NewModel = New
 
-// Init exists satisfy the tea.Model interface.
+// Init exists to satisfy the tea.Model interface.
 func (m Model) Init() tea.Cmd {
 	return nil
 }
 
-// Update is used to animation the progress bar during transitions. Use
+// Update is used to animate the progress bar during transitions. Use
 // SetPercent to create the command you'll need to trigger the animation.
 //
 // If you're rendering with ViewAs you won't need this.
@@ -212,8 +211,7 @@ func (m Model) Update(msg tea.Msg) (tea.Model, tea.Cmd) {
 		}
 
 		// If we've more or less reached equilibrium, stop updating.
-		dist := math.Abs(m.percentShown - m.targetPercent)
-		if dist < 0.001 && m.velocity < 0.01 {
+		if !m.IsAnimating() {
 			return m, nil
 		}
 
@@ -267,7 +265,7 @@ func (m *Model) DecrPercent(v float64) tea.Cmd {
 	return m.SetPercent(m.Percent() - v)
 }
 
-// View renders the an animated progress bar in its current state. To render
+// View renders an animated progress bar in its current state. To render
 // a static progress bar based on your own calculations use ViewAs instead.
 func (m Model) View() string {
 	return m.ViewAs(m.percentShown)
@@ -300,10 +298,15 @@ func (m Model) barView(b *strings.Builder, percent float64, textWidth int) {
 	if m.useRamp {
 		// Gradient fill
 		for i := 0; i < fw; i++ {
-			if m.scaleRamp {
-				p = float64(i) / float64(fw)
+			if fw == 1 {
+				// this is up for debate: in a gradient of width=1, should the
+				// single character rendered be the first color, the last color
+				// or exactly 50% inbetween? I opted for 50%
+				p = 0.5
+			} else if m.scaleRamp {
+				p = float64(i) / float64(fw-1)
 			} else {
-				p = float64(i) / float64(tw)
+				p = float64(i) / float64(tw-1)
 			}
 			c := m.rampColorA.BlendLuv(m.rampColorB, p).Hex()
 			b.WriteString(termenv.
@@ -337,7 +340,7 @@ func (m Model) percentageView(percent float64) string {
 func (m *Model) setRamp(colorA, colorB string, scaled bool) {
 	// In the event of an error colors here will default to black. For
 	// usability's sake, and because such an error is only cosmetic, we're
-	// ignoring the error for sake of usability.
+	// ignoring the error.
 	a, _ := colorful.Hex(colorA)
 	b, _ := colorful.Hex(colorB)
 
@@ -358,6 +361,12 @@ func max(a, b int) int {
 	return b
 }
 
+// IsAnimating returns false if the progress bar reached equilibrium and is no longer animating.
+func (m *Model) IsAnimating() bool {
+	dist := math.Abs(m.percentShown - m.targetPercent)
+	return !(dist < 0.001 && m.velocity < 0.01)
+}
+
 func min(a, b int) int {
 	if a < b {
 		return a
