@@ -68,7 +68,7 @@ func (m Model) Init() tea.Cmd {
 	return nil
 }
 
-// AtTop returns whether or not the viewport is in the very top position.
+// AtTop returns whether or not the viewport is at the very top position.
 func (m Model) AtTop() bool {
 	return m.YOffset <= 0
 }
@@ -147,8 +147,7 @@ func (m *Model) ViewDown() []string {
 		return nil
 	}
 
-	m.SetYOffset(m.YOffset + m.Height)
-	return m.visibleLines()
+	return m.LineDown(m.Height)
 }
 
 // ViewUp moves the view up by one height of the viewport. Basically, "page up".
@@ -157,8 +156,7 @@ func (m *Model) ViewUp() []string {
 		return nil
 	}
 
-	m.SetYOffset(m.YOffset - m.Height)
-	return m.visibleLines()
+	return m.LineUp(m.Height)
 }
 
 // HalfViewDown moves the view down by half the height of the viewport.
@@ -167,8 +165,7 @@ func (m *Model) HalfViewDown() (lines []string) {
 		return nil
 	}
 
-	m.SetYOffset(m.YOffset + m.Height/2)
-	return m.visibleLines()
+	return m.LineDown(m.Height / 2)
 }
 
 // HalfViewUp moves the view up by half the height of the viewport.
@@ -177,13 +174,12 @@ func (m *Model) HalfViewUp() (lines []string) {
 		return nil
 	}
 
-	m.SetYOffset(m.YOffset - m.Height/2)
-	return m.visibleLines()
+	return m.LineUp(m.Height / 2)
 }
 
 // LineDown moves the view down by the given number of lines.
 func (m *Model) LineDown(n int) (lines []string) {
-	if m.AtBottom() || n == 0 {
+	if m.AtBottom() || n == 0 || len(m.lines) == 0 {
 		return nil
 	}
 
@@ -191,20 +187,38 @@ func (m *Model) LineDown(n int) (lines []string) {
 	// greater than the number of lines we actually have left before we reach
 	// the bottom.
 	m.SetYOffset(m.YOffset + n)
-	return m.visibleLines()
+
+	// Gather lines to send off for performance scrolling.
+	bottom := clamp(m.YOffset+m.Height, 0, len(m.lines))
+	top := clamp(m.YOffset+m.Height-n, 0, bottom)
+	return m.lines[top:bottom]
 }
 
 // LineUp moves the view down by the given number of lines. Returns the new
 // lines to show.
 func (m *Model) LineUp(n int) (lines []string) {
-	if m.AtTop() || n == 0 {
+	if m.AtTop() || n == 0 || len(m.lines) == 0 {
 		return nil
 	}
 
 	// Make sure the number of lines by which we're going to scroll isn't
 	// greater than the number of lines we are from the top.
 	m.SetYOffset(m.YOffset - n)
-	return m.visibleLines()
+
+	// Gather lines to send off for performance scrolling.
+	top := max(0, m.YOffset)
+	bottom := clamp(m.YOffset+n, 0, m.maxYOffset())
+	return m.lines[top:bottom]
+}
+
+// TotalLineCount returns the total number of lines (both hidden and visible) within the viewport.
+func (m Model) TotalLineCount() int {
+	return len(m.lines)
+}
+
+// VisibleLineCount returns the number of the visible lines within the viewport.
+func (m Model) VisibleLineCount() int {
+	return len(m.visibleLines())
 }
 
 // GotoTop sets the viewport to the top position.
@@ -240,9 +254,8 @@ func Sync(m Model) tea.Cmd {
 // number of lines. Use Model.ViewDown to get the lines that should be rendered.
 // For example:
 //
-//     lines := model.ViewDown(1)
-//     cmd := ViewDown(m, lines)
-//
+//	lines := model.ViewDown(1)
+//	cmd := ViewDown(m, lines)
 func ViewDown(m Model, lines []string) tea.Cmd {
 	if len(lines) == 0 {
 		return nil
@@ -319,17 +332,17 @@ func (m Model) updateAsModel(msg tea.Msg) (Model, tea.Cmd) {
 		}
 
 	case tea.MouseMsg:
-		if !m.MouseWheelEnabled {
+		if !m.MouseWheelEnabled || msg.Action != tea.MouseActionPress {
 			break
 		}
-		switch msg.Type {
-		case tea.MouseWheelUp:
+		switch msg.Button {
+		case tea.MouseButtonWheelUp:
 			lines := m.LineUp(m.MouseWheelDelta)
 			if m.HighPerformanceRendering {
 				cmd = ViewUp(m, lines)
 			}
 
-		case tea.MouseWheelDown:
+		case tea.MouseButtonWheelDown:
 			lines := m.LineDown(m.MouseWheelDelta)
 			if m.HighPerformanceRendering {
 				cmd = ViewDown(m, lines)
@@ -360,9 +373,10 @@ func (m Model) View() string {
 	contentWidth := w - m.Style.GetHorizontalFrameSize()
 	contentHeight := h - m.Style.GetVerticalFrameSize()
 	contents := lipgloss.NewStyle().
+		Width(contentWidth).      // pad to width.
 		Height(contentHeight).    // pad to height.
 		MaxHeight(contentHeight). // truncate height if taller.
-		MaxWidth(contentWidth).   // truncate width.
+		MaxWidth(contentWidth).   // truncate width if wider.
 		Render(strings.Join(m.visibleLines(), "\n"))
 	return m.Style.Copy().
 		UnsetWidth().UnsetHeight(). // Style size already applied in contents.
